#include "databaseworker.h"

#include <QStandardPaths>
#include <QDir>
#include <QSqlError>
#include <QSqlRecord>
#include <QDateTime>
#include <QDebug>
#include <QTextStream>
#include <libintl.h>
#include <QLocale>

#include "../constants.h"

DatabaseWorker* DatabaseWorker::s_instance = 0;

DatabaseWorker::DatabaseWorker(QObject *parent)
    : QObject(parent)
    , m_databaseOpen(false)
    , m_SqlTablePump(new QSqlQueryModel())
    , m_SqlTableEntries(new QSqlQueryModel())
{
    openDatabase();
    checkTables();
}
DatabaseWorker::~DatabaseWorker(){
    closeDatabase();
}
DatabaseWorker *DatabaseWorker::instance()
{
    if (!s_instance) {
        s_instance = new DatabaseWorker();
    }
    return s_instance;
}
void DatabaseWorker::closeDatabase(){
    m_SqlTablePump->clear();
    m_SqlTableEntries->clear();
    if(m_db.isOpen()){
        m_db.close();
    }
}
void DatabaseWorker::updateDatabaseV3(){
    int version = databaseVersion();
    if (version == 2){
        m_query->prepare(QUERY_UPDATE_VERSION);
        m_query->addBindValue(3);
        m_query->exec();
    }
}
QVariant DatabaseWorker::getSetting(QString setting){
    if(!setting.isEmpty()){
        m_query->prepare(QUERY_GET_SETTING);
        m_query->addBindValue(setting);
        m_query->exec();
        m_query->next();

        return m_query->record().value(0);
    }
    return QVariant();
}
bool DatabaseWorker::setSetting(QString setting, QString value){
    if(!setting.isEmpty()){
        m_query->prepare(QUERY_UPDATE_SETTING);
        m_query->addBindValue(setting);
        m_query->addBindValue(value);
        if (!m_query->exec()){
            qDebug() << "Fehler beim Speichern von " + setting;
            qDebug() << m_query->lastError();
            return false;
        }
        return true;
    }
    return false;
}

void DatabaseWorker::openDatabase(){
    QDir databaseDir(QStandardPaths::standardLocations(QStandardPaths::DataLocation).first());
    QString dbName = DATABASENAME;
    if(!databaseDir.exists()){
        databaseDir.mkdir(databaseDir.absolutePath());
    }
    if(!m_db.connectionNames().contains(dbName)){
        m_db = QSqlDatabase::addDatabase("QSQLITE",dbName);
        m_db.setDatabaseName(databaseDir.absolutePath()+"/" + dbName + ".sqlite");
    }
    else{
        m_db = QSqlDatabase::database(dbName);
    }

    if (!m_db.isOpen()){
        qWarning() << "Worker: Database: " + m_db.databaseName() + " is not open!";
        if (!m_db.open()){
            qWarning() << "Worker:Error: " + m_db.lastError().text();
            m_databaseOpen = false;
        } else {
            qWarning() << "Worker: Database: " + m_db.databaseName() + " opened!";
            m_databaseOpen = true;
        }
    }
    m_query = new QSqlQuery(m_db);
}

bool DatabaseWorker::deleteAllEntries(){

    bool ok = m_query->exec(QUERY_DELETE_TABLE_ENTRIES);
    if (!ok){
        qWarning() << "Worker: Fehler beim Löschen der DB!";
        qWarning() << m_query->lastError();
        return false;
    }
    ok = m_query->exec(QUERY_CREATE_TABLE_ENTRIES);
    if (!ok){
        qWarning() << "Worker: Fehler beim Erzeugen des Table!";
        qWarning() << m_query->lastError();
        return false;
    }

    return true;
}
bool DatabaseWorker::deleteEntry(const int &id){

    m_query->prepare(QUERY_ENTRIES_DELETE_BY_ID);
    m_query->addBindValue(id);
    if (!m_query->exec()){
        qDebug() << "Fehler beim Löschen der ID " + QString::number(id);
        qDebug() << m_query->lastError();
        return false;
    }

    return true;
}

QSqlQueryModel* DatabaseWorker::getAllEntries(){

    m_SqlTableEntries->setQuery(QUERY_GET_ALL_ENTRIES_ORDER_BY_DATE, m_db);
    if (m_SqlTableEntries->lastError().isValid()){
        qWarning() << m_SqlTableEntries->lastError();
        m_SqlTableEntries->clear();
    }

    return m_SqlTableEntries;
}
QSqlQueryModel* DatabaseWorker::getAllPumpRates(){

    m_SqlTablePump->clear();
    m_SqlTablePump->setQuery(QUERY_GET_ALL_PUMPBASAL, m_db);
    if (m_SqlTablePump->lastError().isValid()){
        qWarning() << m_SqlTablePump->lastError();
        m_SqlTablePump->clear();
    }

    return m_SqlTablePump;
}
double DatabaseWorker::columnSum(QString colName){

    if (colName.operator ==("bolus")) m_query->exec(QUERY_SUM_BOLUS);
    else if (colName.operator ==("basal")) m_query->exec(QUERY_SUM_BASAL);
    else return -1;
    m_query->next();

    return m_query->record().value(0).toDouble();
}

int DatabaseWorker::databaseVersion(){

    m_query->exec(QUERY_GET_VERSION);
    m_query->next();

    return m_query->record().value(0).toInt();
}
void DatabaseWorker::checkTables(){
    QStringList tableNames = m_db.tables(QSql::Tables);

    if (tableNames.indexOf("entries") == -1){
        m_query->prepare(QUERY_CREATE_TABLE_ENTRIES);
        m_query->exec();
    }
    if (tableNames.indexOf("version") == -1){
        m_query->prepare(QUERY_CREATE_TABLE_VERSION);
        m_query->exec();
        m_query->prepare("INSERT INTO version VALUES (2)");
        m_query->exec();
    }
    if (tableNames.indexOf("pump_basal_rate") == -1){
        m_query->prepare(QUERY_CREATE_TABLE_PUMPBASAL);
        m_query->exec();
    }
    if (tableNames.indexOf("settings") == -1){
        m_query->prepare(QUERY_CREATE_TABLE_APPSETTINGS);
        m_query->exec();
        setSetting(VARNAME_SETTINGS_MEALCARBS_CONVVALUE,"10");
        setSetting(VARNAME_SETTINGS_SELECTEDMEALCARBSGRAMS,"FALSE");
    }
}
bool DatabaseWorker::deletePumpEntry(const int &id){

    m_query->prepare(QUERY_PUMPBASAL_DELETE_BY_ID);
    m_query->addBindValue(id);
    if (!m_query->exec()){
        qWarning() << "Fehler beim Löschen der Pump-ID " + QString::number(id);
        qWarning() << m_query->lastError();
        return false;
    }

    return true;
}
int DatabaseWorker::savePumpEntry(PumpBasal *pumpEntry){
    if(m_databaseOpen) return pumpToDatabase(pumpEntry, false, true);
    else return -1;
}
int DatabaseWorker::updatePumpEntry(PumpBasal *pumpEntry){
    if(m_databaseOpen) return pumpToDatabase(pumpEntry, true, false);
    else return -1;
}
int DatabaseWorker::pumpToDatabase(PumpBasal* pumpEntry, const bool bUpdate, const bool bsendId){

    if(!bUpdate) m_query->prepare(QUERY_PUMPBASAL_INSERT);
    else m_query->prepare(QUERY_PUMPBASAL_UPDATE_BY_ID);
    m_query->addBindValue(pumpEntry->name());
    m_query->addBindValue(pumpEntry->basalRate());
    m_query->addBindValue(pumpEntry->isActive());
    if (bUpdate) m_query->addBindValue(pumpEntry->id());
    if (!m_query->exec()){
        qWarning() << m_query->lastError();
        return -1;
    }

    if (!bUpdate && bsendId) return m_query->lastInsertId().toInt();

    return 1;
}

int DatabaseWorker::saveEntry(DiaryEntry* entry){
    if(m_databaseOpen) return entryToDatabase(entry, false, true);
    else return -1;
}
int DatabaseWorker::updateEntry(DiaryEntry *entry){
    if(m_databaseOpen) return entryToDatabase(entry, true, false);
    else return -1;
}
int DatabaseWorker::importEntry(DiaryEntry *entry){
    if(m_databaseOpen) return entryToDatabase(entry,false, false);
    else return -1;
}
int DatabaseWorker::entryToDatabase(DiaryEntry* entry, const bool bUpdate, const bool bsendId){

    if (!bUpdate) m_query->prepare(QUERY_ENTRIES_INSERT);
    else m_query->prepare(QUERY_ENTRIES_UPDATE_BY_ID);

    m_query->addBindValue(entry->createdTimeStamp());
    m_query->addBindValue(entry->createdDate());
    m_query->addBindValue(entry->createdTimeStamp().toLocalTime());

    if (entry->glucMeasValueMg() != 0) m_query->addBindValue(entry->glucMeasValueMg());
    else m_query->addBindValue(SQLNULL);
    if (entry->glucMeasValueMmol() != 0) m_query->addBindValue(entry->glucMeasValueMmol());
    else m_query->addBindValue(SQLNULL);
    if (entry->mealCarbs() != 0) m_query->addBindValue(entry->mealCarbs());
    else m_query->addBindValue(SQLNULL);
    if (entry->bolus() != 0) m_query->addBindValue(entry->bolus());
    else m_query->addBindValue(SQLNULL);
    if (!entry->temporaryPumpBasal().isEmpty()) m_query->addBindValue(entry->temporaryPumpBasal());
    else m_query->addBindValue(SQLNULL);
    if (entry->basalInjection() != 0) m_query->addBindValue(entry->basalInjection());
    else m_query->addBindValue(SQLNULL);
    if (!entry->note().isEmpty()) m_query->addBindValue(entry->note());
    else m_query->addBindValue(SQLNULL);

    if (bUpdate) m_query->addBindValue(entry->id());

    if (!m_query->exec()){
        qWarning() << m_query->lastError();
        return -1;
    }

    if (!bUpdate && bsendId) return m_query->lastInsertId().toInt();

    return 1;
}
QString DatabaseWorker::exportToCsv(const bool selectCarbGram, const int convValue){
    QDateTime dbEntryTime;
    QStringList header;
    QDir m_fileDir;
    QString m_fileName;
    QString m_createdDate;
    m_SqlTableEntries->clear();
    m_SqlTableEntries->setQuery(QUERY_GET_ALL_ENTRIES_ORDER_BY_DATE, m_db);

    m_fileDir = QStandardPaths::standardLocations(QStandardPaths::CacheLocation).first();
    m_fileDir = m_fileDir.absolutePath();// + QDir::separator() + "Diabetes-Diary-Export";
    if (!m_fileDir.exists()){
        m_fileDir.mkdir(m_fileDir.absolutePath());
    }
    m_createdDate = QDate::currentDate().toString("yyyy_MM_dd");
    m_fileName = "Diabetes_Diary_export_" + m_createdDate;

    m_fileName = m_fileDir.absolutePath() + QDir::separator() + m_fileName + ".csv";
    qWarning() << "CSV-Export to " + m_fileName;
    QFile saveFile(m_fileName);

    if(!saveFile.open(QFile::WriteOnly | QFile::Truncate)){
        qWarning() << "not open: " + saveFile.error();
        return QString();
    }
    //write Header
    QString carbHeader = gettext("Carbs");
    if (selectCarbGram) carbHeader = carbHeader + " (g)";
    header << "\"" << gettext("Date") << "\",\"" << gettext("Time") << "\",\"" << gettext("Blood glucose (mg/dl)") << "\",\"" << gettext("Blood glucose (mmol/l)") << "\",\""
           << carbHeader << "\",\"" << gettext("Bolus") << "\",\"" << gettext("Temporary basal rate") << "\",\""
           << gettext("Basal (Pen)") << "\",\"" << gettext("Note") << "\"";

    QTextStream stream(&saveFile);
    for(int i=0;i<header.size();i++){
        stream << header.at(i);
    }
    stream << "\n";
    while(m_SqlTableEntries->canFetchMore()){
        m_SqlTableEntries->fetchMore();
    }
    //write data
    for (int i=0;i < m_SqlTableEntries->rowCount();i++){
        dbEntryTime = m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_CREATEDTIME)).toDateTime();

        QString mealCarbs = m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_MEALCARBS)).toString();
        if (selectCarbGram && !mealCarbs.isEmpty()) mealCarbs = QString::number(mealCarbs.toDouble() * convValue,'f',0);

        stream << "\""+m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_DATE)).toDate().toString("dd.MM.yyyy") + "\",\"" +
                dbEntryTime.time().toString(Qt::SystemLocaleShortDate) + "\";\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_GLUCOSE_MG)).toString() + "\",\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_GLUCOSE_MMOL)).toString() + "\",\"" +
                mealCarbs + "\",\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_BOLUS)).toString() + "\",\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_TEMP_PUMPBASAL)).toString() + "\",\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_BASALINJECT)).toString() + "\",\"" +
                m_SqlTableEntries->data(m_SqlTableEntries->index(i,IDX_ENTRIES_NOTE)).toString().replace("\n"," ");
        stream << "\"\n";
    }
    saveFile.close();
    return m_fileName;
}


