#include <QDir>
#include <QStandardPaths>
#include <QDebug>
#include <QSqlError>
#include <QSqlQuery>
#include <QDateTime>

#include "diarystatistics.h"
#include "../constants.h"

#include "../Database/databaseworker.h"

DiaryStatistics::DiaryStatistics(QObject *parent)
    : QObject(parent)
    , m_model(new QSqlQueryModel)
    , m_numberOfEntries(0)
{

}

void DiaryStatistics::readSqlData(){
    m_model = DatabaseWorker::instance()->getAllEntries();
    if (m_model->lastError().isValid()){
        qDebug() << m_model->lastError();
    }
    while (m_model->canFetchMore()) m_model->fetchMore();
}
void DiaryStatistics::setStatistics(){
    readSqlData();
    bool selectedCarbGrams = DatabaseWorker::instance()->getSetting(VARNAME_SETTINGS_SELECTEDMEALCARBSGRAMS).toBool();
    quint8 convCarbValue = DatabaseWorker::instance()->getSetting(VARNAME_SETTINGS_MEALCARBS_CONVVALUE).toInt();

    //number of measurements
    m_numberOfEntries = 0;
    for (int rowIdx = 0; rowIdx < m_model->rowCount(); rowIdx++){
        QVariant entry = m_model->data(m_model->index(rowIdx,IDX_ENTRIES_GLUCOSE_MG));
        if (!entry.isNull()) m_numberOfEntries++;
    }
    Q_EMIT numberOfEntriesChanged();
    //Average
    if (m_measurementUnit.operator ==("mg/dl")) {
        m_measurementAverageWeek    = columnAverage(IDX_ENTRIES_GLUCOSE_MG,"Week");
        m_measurementAverageMonth   = columnAverage(IDX_ENTRIES_GLUCOSE_MG,"Month");
        m_measurementAverageQuarter = columnAverage(IDX_ENTRIES_GLUCOSE_MG,"Quarter");

        m_hba1c = m_measurementAverageQuarter + 86;
        m_hba1c = m_hba1c / 33.3;
    }
    else {
        m_measurementAverageWeek    = columnAverage(IDX_ENTRIES_GLUCOSE_MMOL,"Week");
        m_measurementAverageMonth   = columnAverage(IDX_ENTRIES_GLUCOSE_MMOL,"Month");
        m_measurementAverageQuarter = columnAverage(IDX_ENTRIES_GLUCOSE_MMOL,"Quarter");

        m_hba1c = (m_measurementAverageQuarter * MEASUREMENT_CONV_FACTOR) + 86;
        m_hba1c = m_hba1c / 33.3;
    }
    Q_EMIT measurementAverageChanged();
    Q_EMIT hba1cChanged();

    QDate currentDate = QDate::currentDate();
    int quarterDays = abs(currentDate.daysTo(currentDate.addMonths(-3)));

    m_insulinAverageWeek = columnSum(IDX_ENTRIES_BOLUS,7) / 7;
    m_insulinAverageMonth = columnSum(IDX_ENTRIES_BOLUS,30) / 30;
    m_insulinAverageQuarter = columnSum(IDX_ENTRIES_BOLUS,quarterDays) / quarterDays;
    Q_EMIT insulinAverageChanged();

    if (selectedCarbGrams){
        m_carbsAverageWeek = columnSum(IDX_ENTRIES_MEALCARBS,7) * convCarbValue / 7;
        m_carbsAverageMonth = columnSum(IDX_ENTRIES_MEALCARBS,30) * convCarbValue / 30;
        m_carbsAverageQuarter = columnSum(IDX_ENTRIES_MEALCARBS,quarterDays) * convCarbValue / quarterDays;
    }
    else {
        m_carbsAverageWeek = columnSum(IDX_ENTRIES_MEALCARBS,7) / 7;
        m_carbsAverageMonth = columnSum(IDX_ENTRIES_MEALCARBS,30) / 30;
        m_carbsAverageQuarter = columnSum(IDX_ENTRIES_MEALCARBS,quarterDays) / quarterDays;
    }
    Q_EMIT carbsAverageChanged();

    if(!m_diabetesTherapy){
        double bolusSum = DatabaseWorker::instance()->columnSum("bolus");
        double basalSum = DatabaseWorker::instance()->columnSum("basal");

        m_bolusRatio = bolusSum / (bolusSum+basalSum) * 100;
        m_basalRatio = basalSum / (bolusSum+basalSum) * 100;
    }
    else{
        m_bolusRatio = 0;
        m_basalRatio = 0;
    }
    Q_EMIT ratioChanged();
}

double DiaryStatistics::columnAverage(const int &columnIdx, const QString &duration){
    QDate dateNow = QDate::currentDate();
    QDate dateDuration = dateNow;
    QDate dbDate = dateNow;
    int count = 0;
    double countSum = 0;

    if (duration == "Week") dateDuration=dateDuration.addDays(-7);
    else if (duration == "Month") dateDuration=dateDuration.addMonths(-1);
    else if (duration == "Quarter") dateDuration=dateDuration.addMonths(-3);
    else dateDuration=dateDuration.addYears(-3); //Default

    QVariant value = SQLNULL;
    for (int i = 0; i < m_model->rowCount(); i++){
        dbDate = m_model->data(m_model->index(i,2)).toDate();
        if (dbDate.operator <=(dateNow) & dbDate.operator >(dateDuration)){
            value = m_model->data(m_model->index(i,columnIdx));
            if (!value.isNull()){
                countSum += m_model->data(m_model->index(i,columnIdx)).toDouble();
                count++;
            }
        }
    }
    if (count > 0) {
        return countSum/count;
    }
    else return 0;
}
double DiaryStatistics::columnSum(const int &columnIdx, const int &days){
    QDate dateNow = QDate::currentDate();
    QDate dateDuration = dateNow;
    QDate dbDate = dateNow;
    double countSum = 0;
    bool between = true;

    if (days >= 0) dateDuration = dateDuration.addDays(-days);
    else between = false;

    for (int i = 0; i < m_model->rowCount(); i++){
        dbDate = m_model->data(m_model->index(i,2)).toDate();
        if (between){
            if (dbDate.operator <=(dateNow) & dbDate.operator >(dateDuration)){
                countSum += m_model->data(m_model->index(i,columnIdx)).toDouble();
            }
        }
        else if (!between) countSum += m_model->data(m_model->index(i,columnIdx)).toDouble();
    }
    return countSum;
}
void DiaryStatistics::measurementOutsideRefresh(const double &minValue, const double &maxValue){
    QVariant value;
    int columnIdx;

    m_measurementAbove = 0;
    m_measurementBelow = 0;

    if (m_measurementUnit.operator ==("mg/dl")) {
        columnIdx = IDX_ENTRIES_GLUCOSE_MG;
    }
    else columnIdx = IDX_ENTRIES_GLUCOSE_MMOL;

    for (int i = 0; i < m_model->rowCount(); i++){
        value = m_model->data(m_model->index(i,columnIdx));
        if (!value.isNull()){
            if(value.toDouble() > maxValue) {
                m_measurementAbove++;
            }
            else if (value.toDouble() < minValue){
                m_measurementBelow++;
            }
        }
    }
    m_measurementAbove = m_measurementAbove / m_numberOfEntries * 100;
    m_measurementBelow = m_measurementBelow / m_numberOfEntries * 100;
    m_measurementOutside = m_measurementAbove + m_measurementBelow;

    Q_EMIT measurementOutsideChanged();
}



QString DiaryStatistics::measurementUnit(){
    return m_measurementUnit;
}
void DiaryStatistics::setMeasurementUnit(const QString &u){
    if (m_measurementUnit != u){
        m_measurementUnit = u;
        Q_EMIT measurementUnitChanged();
    }
}
bool DiaryStatistics::diabetesTherapy(){
    return m_diabetesTherapy;
}
void DiaryStatistics::setDiabetesTherapy(const bool &therapy){
    if (m_diabetesTherapy != therapy){
        m_diabetesTherapy = therapy;
        Q_EMIT diabetesTherapyChanged();
    }
}
quint64 DiaryStatistics::numberOfEntries(){
    return m_numberOfEntries;
}
double DiaryStatistics::measurementAverageWeek(){
    return m_measurementAverageWeek;
}
double DiaryStatistics::measurementAverageMonth(){
    return m_measurementAverageMonth;
}
double DiaryStatistics::measurementAverageQuarter(){
    return m_measurementAverageQuarter;
}
double DiaryStatistics::measurementAbove(){
    return m_measurementAbove;
}
double DiaryStatistics::measurementBelow(){
    return m_measurementBelow;
}
double DiaryStatistics::measurementOutside(){
    return m_measurementOutside;
}
double DiaryStatistics::hba1c(){
    return m_hba1c;
}
double DiaryStatistics::basalRatio(){
    return m_basalRatio;
}
double DiaryStatistics::bolusRatio(){
    return m_bolusRatio;
}
double DiaryStatistics::insulinAverageWeek(){
    return m_insulinAverageWeek;
}
double DiaryStatistics::insulinAverageMonth(){
    return m_insulinAverageMonth;
}
double DiaryStatistics::insulinAverageQuarter(){
    return m_insulinAverageQuarter;
}
double DiaryStatistics::carbsAverageWeek(){
    return m_carbsAverageWeek;
}
double DiaryStatistics::carbsAverageMonth(){
    return m_carbsAverageMonth;
}
double DiaryStatistics::carbsAverageQuarter(){
    return m_carbsAverageQuarter;
}

